#include "stdafx.h"
#include "RotBlackout.h"
#include "misc.h"

#define SECONDS_IN_A_DAY (60*60*24)
#define SECONDS_IN_A_HOUR (60*60)
#define SECONDS_IN_A_MINUTE (60)
#define FROM100NS_TO_1SEC 10000000

RotBlackout::RotBlackout()
	: PrivateProfile( _T(".dat") )
	, day_start( 0 )
	, cycle( 0 )
	, total( 0 )
{
}

RotBlackout::~RotBlackout()
{
}

inline int RotBlackout::CalcSec( HOURMIN hm )
{
	return hm.hour * SECONDS_IN_A_HOUR + hm.minute * SECONDS_IN_A_MINUTE;
}

BOOL RotBlackout::Load( TCHAR *filename )
{
	BOOL bRet = FALSE;
	FILE *fpTable = NULL;
	char line[256], *p;
	int year, month, day, hour0, minute0, hour1, minute1;
	//struct tm tm;
	SYSTEMTIME st;
	FILETIME ft;
	ULARGE_INTEGER ui;

	if ( filename ) {
		_tcscpy_s( profName, _MAX_PATH, filename );
	}

	total = cycle = 0;

	do {
		if ( _tfopen_s( &fpTable, profName, _T("rt") ) ) {
			break;
		}

		fgets( line, 256, fpTable );
		if ( sscanf_s( line, "%d/%d/%d", &year, &month, &day ) != 3 ) {
			break;
		}
		memset( &st, 0, sizeof(st) );
		st.wYear = year;
		st.wMonth = month;
		st.wDay = day;
		SystemTimeToFileTime( &st, &ft );
		ui.LowPart = ft.dwLowDateTime;
		ui.HighPart = ft.dwHighDateTime;
		day_start = ui.QuadPart;

		ULONGLONG ut = GetLocalTime64();

		while( fgets( line, 256, fpTable ) != NULL ) {
			p = line;
			do {
				if ( *p == ',' ) {
					p++;
				}
				if ( sscanf_s( p, "%d:%d-%d:%d", &hour0, &minute0, &hour1, &minute1 ) == 4 ) {

					BLACKOUT_ELEMENT be;
					be.idx = cycle;
					be.start.hour = hour0;
					be.start.minute = minute0;
					be.end.hour = hour1;
					be.end.minute = minute1;
					table.push_back( be );
					total++;
				}
			} while( ( p = strstr( p, "," )  ) != NULL );
			cycle++;
		}

	} while( FALSE );

	if ( fpTable ) {
		fclose( fpTable );
		fpTable = NULL;
	}
	if ( total ) {
		bRet = TRUE;
	}

	return bRet;
}

BOOL RotBlackout::IsIn( SYSTEMTIME *endtime )
{
	//struct tm tm;
	ULONGLONG current;
	int idx, sday;
	BOOL bRet = FALSE;

	if ( total == 0 ) {
		return FALSE;
	}

	current = GetLocalTime64();

	sday = (int)( ( current - day_start ) / FROM100NS_TO_1SEC );
	idx = ( sday / SECONDS_IN_A_DAY ) % cycle;
	sday = sday % SECONDS_IN_A_DAY;

	for ( int c = 0; c < total; c++ ) {

		if ( table[c].idx == idx ) {

			if ( ( CalcSec( table[c].start ) <= sday ) && ( CalcSec( table[c].end ) >= sday ) ) {
				
				bRet = TRUE;

				if ( endtime ) {
					GetLocalTime( endtime );
					endtime->wHour = table[c].end.hour;
					endtime->wMinute = table[c].end.minute;
					endtime->wSecond = endtime->wMilliseconds = 0;
				}
				break;
			}
		}
	}

	return bRet;
}
