#include "StdAfx.h"
#include "Setting.h"
#include "SettingDlg.h"
#include "resource.h"
#include <wincrypt.h>
#include <mstask.h>
#include "misc.h"

#define ReleaseCom(x) if(x){x->Release();x=NULL;}

#define SECONDS_IN_A_DAY (60*60*24)
#define SECONDS_IN_A_HOUR (60*60)
#define SECONDS_IN_A_MINUTE (60)
#define FROM100NS_TO_1SEC 10000000

#define USER_KEYNAME		_T("user")
#define PASSWORD_KEYNAME	_T("password")
#define INTERVAL_DEFAULT	60
#define INTERVAL_KEYNAME	_T("interval")
#define COUNT_DEFAULT		5
#define COUNT_KEYNAME		_T("count")
#define DELAY_DEFAULT		60
#define DELAY_KEYNAME		_T("delay")
#define PLANNED_DEFAULT		0
#define PLANNED_KEYNAME		_T("planned")
#define SLEEP_DEFAULT		0
#define SLEEP_KEYNAME		_T("sleep")
#define OFFPEAK_DEFAULT		0
#define OFFPEAK_KEYNAME		_T("offpeak")

Setting::Setting(void)
	: PrivateProfile( _T(".ini") )
{
	settings.user.clear();
	settings.pass.clear();
	settings.interval = 0;
	settings.count = 0;
	settings.delay = 0;
	settings.planned = FALSE;
	settings.sleep = FALSE;
	settings.offpeak = FALSE;
}

Setting::~Setting(void)
{
	Save();
}

BOOL Setting::EncryptPass( tstring &org, tstring &dest )
{
	BOOL ret;
	DATA_BLOB raw, dec;
	int length = org.length() * 5 + 1;

#ifdef _UNICODE
	BYTE *bin = new BYTE[length];
	int num;

	num = WideCharToMultiByte( CP_UTF8, 0, org.c_str(), org.length(), (LPSTR)bin, length, NULL, NULL );
	bin[num] = 0;

	raw.cbData = num;
	raw.pbData = bin;
#else
	raw.cbData = org.length();
	raw.pbData = (BYTE*)org.c_str();
#endif

	ret = CryptProtectData( &raw, NULL, NULL, NULL, NULL, CRYPTPROTECT_UI_FORBIDDEN, &dec );

	if ( ret ) {
		dest.clear();
		for ( DWORD c = 0; c < dec.cbData; c++ ) {
			TCHAR hex[3];
			_stprintf_s( hex, 3, _T("%02x"), dec.pbData[c] );
			dest += hex;
		}
		LocalFree( dec.pbData );
	}

#ifdef _UNICODE
	delete[] bin;
#endif

	return ret;
}

BOOL Setting::DecryptPass( tstring &enc, tstring &dest )
{
	BOOL ret;
	DATA_BLOB raw, dec;
	int length = enc.length() / 2;
	BYTE *bin = new BYTE[length];

	for ( int c = 0; c < length; c++ ) {
		unsigned int t;
		_stscanf_s( enc.substr( c * 2, 2 ).c_str(), _T("%x"), &t );
		bin[c] = (unsigned char)t;
	}

	raw.cbData = length;
	raw.pbData = bin;
	ret = CryptUnprotectData( &raw, NULL, NULL, NULL, NULL, CRYPTPROTECT_UI_FORBIDDEN, &dec );

	if ( ret ) {
#ifdef _UNICODE
		length = dec.cbData + 1;
		wchar_t *t = new wchar_t[length];
		int num = MultiByteToWideChar( CP_UTF8, 0, (LPCSTR)dec.pbData, dec.cbData, t, length );
		t[num] = 0;
		dest = t;
		delete[] t;
#else
		dest.assign( (char*)dec.pbData, dec.cbData );
#endif
		LocalFree( dec.pbData );
	}

	delete[] bin;

	return ret;
}

BOOL Setting::WritePass( const TCHAR *value, TCHAR *def, TCHAR *key )
{
	BOOL bRet = TRUE;
	tstring tstr0, tstr1;
	TCHAR temp[2048];
	std::string ustr0, ustr1;

	GetPrivateProfileString( appName, key, def, temp, 2048, profName );
	tstr0 = temp;
	if ( tstr0.length() != 0 ) {
		DecryptPass( tstr0, tstr1 );
	}

	if ( tstr1 != value ) {
		tstr0 = value;
		EncryptPass( tstr0, tstr1 );
		WriteString( tstr1.c_str(), _T(""), PASSWORD_KEYNAME );
	}

	return bRet;
}

void Setting::Load(void)
{
	TCHAR temp[2048];
	tstring tstr;

	GetPrivateProfileString( appName, USER_KEYNAME,		_T(""), temp, 2048, profName );
	settings.user = temp;
	GetPrivateProfileString( appName, PASSWORD_KEYNAME,	_T(""), temp, 2048, profName );
	tstr = temp;
	if ( tstr.length() != 0 ) {
		DecryptPass( tstr, settings.pass );
	}
	else {
		settings.pass.clear();
	}

	settings.interval	= GetPrivateProfileInt( appName, INTERVAL_KEYNAME,	INTERVAL_DEFAULT,	profName );
	settings.count		= GetPrivateProfileInt( appName, COUNT_KEYNAME,		COUNT_DEFAULT,		profName );
	settings.delay		= GetPrivateProfileInt( appName, DELAY_KEYNAME,		DELAY_DEFAULT,		profName );
	settings.planned	= GetPrivateProfileInt( appName, PLANNED_KEYNAME,	PLANNED_DEFAULT,	profName );
	settings.sleep		= GetPrivateProfileInt( appName, SLEEP_KEYNAME,		SLEEP_DEFAULT,		profName );
	settings.offpeak	= GetPrivateProfileInt( appName, OFFPEAK_KEYNAME,	OFFPEAK_DEFAULT,	profName );
}

BOOL Setting::Save(void)
{
	tstring tstr;

	WriteString( settings.user.c_str(),	_T(""), USER_KEYNAME );
	WritePass( settings.pass.c_str(), _T(""), PASSWORD_KEYNAME );
	WriteInt( settings.interval,	INTERVAL_DEFAULT,	INTERVAL_KEYNAME );
	WriteInt( settings.count,		COUNT_DEFAULT,		COUNT_KEYNAME );
	WriteInt( settings.delay,		DELAY_DEFAULT,		DELAY_KEYNAME );
	WriteInt( settings.planned,		PLANNED_DEFAULT,	PLANNED_KEYNAME );
	WriteInt( settings.sleep,		SLEEP_DEFAULT,		SLEEP_KEYNAME );
	WriteInt( settings.offpeak,		OFFPEAK_DEFAULT,	OFFPEAK_KEYNAME );

	return TRUE;
}

BOOL Setting::Compare( SETTINGS *org, SETTINGS *dest )
{
	BOOL bRet = FALSE;

	if ( org->user		!= dest->user )		bRet = TRUE;
	if ( org->pass		!= dest->pass )		bRet = TRUE;
	if ( org->count		!= dest->count )	bRet = TRUE;
	if ( org->interval	!= dest->interval )	bRet = TRUE;
	if ( org->delay		!= dest->delay )	bRet = TRUE;
	if ( org->planned	!= dest->planned )	bRet = TRUE;
	if ( org->sleep		!= dest->sleep )	bRet = TRUE;
	if ( org->offpeak	!= dest->offpeak )	bRet = TRUE;

	return bRet;
}

BOOL Setting::ShowDialog(void)
{
	BOOL bRet = FALSE;

	SETTINGS temp;
	SettingDlg dlg( IDD_SETTING );

	temp = settings;
	dlg.SetParams( &temp );

	if ( dlg.DoModal() == IDOK ) {

		dlg.GetParams( &temp );

		if ( Compare( &settings, &temp ) ) {
			settings = temp;
			bRet = TRUE;
		}
	}
	return bRet;
}

BOOL Setting::CreateTask( SYSTEMTIME *st )
{
	BOOL bRet = FALSE;
	ITaskScheduler *pITS = NULL;
	ITaskTrigger *pITaskTrigger = NULL;
	ITask *pITask = NULL;
	IPersistFile *pIPersistFile = NULL;
	HRESULT hr;

	do {
		hr = CoInitialize( NULL );
		if ( FAILED(hr) ) break;

		hr = CoCreateInstance(	CLSID_CTaskScheduler,
								NULL,
								CLSCTX_INPROC_SERVER,
								IID_ITaskScheduler,
								(LPVOID*)&pITS );
		if ( FAILED(hr) ) break;

		hr = pITS->NewWorkItem( L"BlackoutWakeUp",
								CLSID_CTask,
								IID_ITask,
								(IUnknown**)&pITask );
		if ( HRESULT_CODE(hr) == ERROR_FILE_EXISTS ) {
			pITS->Delete( L"BlackoutWakeUp" );
			hr = pITS->NewWorkItem( L"BlackoutWakeUp",
									CLSID_CTask,
									IID_ITask,
									(IUnknown**)&pITask );
		}
		if ( FAILED(hr) ) break;

		ReleaseCom( pITS );

		hr = pITask->SetApplicationName( L"wscript" );
		hr = pITask->SetParameters( L"//b" );
		hr = pITask->SetAccountInformation( settings.user.c_str(), settings.pass.c_str() );
		pITask->SetFlags( TASK_FLAG_DELETE_WHEN_DONE | TASK_FLAG_SYSTEM_REQUIRED );

		TASK_TRIGGER Trigger;
		memset( &Trigger, 0, sizeof(Trigger) );
		Trigger.cbTriggerSize	= sizeof(Trigger);
		Trigger.TriggerType		= TASK_TIME_TRIGGER_ONCE;
		Trigger.wBeginDay		= st->wDay;
		Trigger.wBeginMonth		= st->wMonth;
		Trigger.wBeginYear		= st->wYear;
		Trigger.wStartMinute	= st->wMinute;
		Trigger.wStartHour		= st->wHour;

		WORD piNewTrigger;
		hr = pITask->CreateTrigger( &piNewTrigger, &pITaskTrigger );
		if ( FAILED(hr) ) break;

		hr = pITaskTrigger->SetTrigger( &Trigger );
		if ( FAILED(hr) ) break;

		hr = pITask->QueryInterface( IID_IPersistFile, (LPVOID*)&pIPersistFile );
		if ( FAILED(hr) ) break;

		hr = pIPersistFile->Save( NULL, TRUE );
		if ( FAILED(hr) ) break;

		bRet = TRUE;

	} while( FALSE );

	ReleaseCom( pITask );
	ReleaseCom( pITaskTrigger );
	ReleaseCom( pIPersistFile );
	ReleaseCom( pITS );
	CoUninitialize();

	return bRet;
}

void Setting::GetFromNow( SYSTEMTIME *st )
{
	ULARGE_INTEGER ui;
	FILETIME ft;

	ui.QuadPart = GetLocalTime64();
	ui.QuadPart += (ULONGLONG)settings.delay * SECONDS_IN_A_MINUTE * FROM100NS_TO_1SEC;

	ft.dwLowDateTime = ui.LowPart;
	ft.dwHighDateTime = ui.HighPart;
	FileTimeToSystemTime( &ft, st );
}
